/* 
NewsFleX offline NNTP news reader
NewsFleX is registered Copyright (C) 1997-2005 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NewsFleX.h"

extern FD_NewsFleX *fdui;

FL_IOPT fl_iopt;


int main(int argc, char *argv[])
{
int a, i;
FILE *lockfile;
struct passwd *userinfo;
char temp[TEMP_SIZE];
/*sigset_t block_alarm;*/
FILE *fptr;
char font_name[4096];
int special_font_flag;
Pixmap p, mask;
unsigned w, h;
char dummy [1024];
     
/*select con io 1 byte at the time*/
setbuf(stdout,NULL);
setbuf(stdin,NULL);

/* set defaults */
debug_flag = 0;
global_posting_wrap_chars = 78;
send_newsflex_id = 1;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
user_name = strsave(userinfo -> pw_name);

#ifdef NO_LOCKFILES
/* test if already a NewsFleX running */
sprintf(temp, "pidof NewsFleX > %s/.NewsFleX/pids", home_dir);
fptr = popen(temp, "w");
pclose(fptr);

sprintf(temp, "%s/.NewsFleX/pids", home_dir);
fptr = fopen(temp, "r");
if(fptr)
	{
	a = fscanf(fptr, "%d %d", &b, &b);
	fclose(fptr);

	if(a == 2)
		{
		fprintf(stderr, "NewsFleX: An other NewsFleX is already running, aborting.\n");

		exit(1);
		}
	}
#else // ! NO_LOCKFILES

/* Locking code based on a suggestion by Michal Nazarewicz, but I use lockf(), not flock() */ 

/* create a lockfile */
sprintf(temp, "%s/.NewsFleX/lockfile", home_dir);
lockfile = fopen(temp, "w");
if(! lockfile)
	{
	fprintf(stderr, "NewsFleX: could not create lockfile %s aborting.\n", temp);

	exit(1);
	}

/* test if file is locked */
i = 0;
while(1)
	{
	a = lockf(fileno(lockfile), F_TEST, 0);
	if(a != -1) break;

	if(errno != EAGAIN)
		{
		fprintf(stderr, "NewsFleX: An other NewsFleX is already running, aborting.\n");

		exit(1);	
		}

	i++;
	if(i == 1000) // 10 S
		{
		fprintf(stderr, "NewsFleX: timeout waiting for lockf() F_TEST, aborting.\n");

		exit(1);		
		}

	usleep(10000); // 10 ms
	}
	
/* lock the file */
i = 0;
while(1)
	{
	a = lockf(fileno(lockfile), F_TLOCK, 0);
	if(a != -1)  break;

	if(errno != EAGAIN)
		{
		perror(" NewsFleX: could not lock lockfile because: "); 

		exit(1);
		}

	i++;
	if(i == 1000) // 10 s
		{
		fprintf(stderr, "NewsFleX: timeout EAGAIN waiting for lockf() F_TLOCK, aborting.\n");

		exit(1);
		} 

	usleep(10000); // 10 ms
	}

/* file stays open, else lock is lost */

#endif // NO_LOCKFILES


/* process command line arguments */
special_font_flag = 0;
font_name[0] = 0;
while(1)
	{
	/* allow for '-shared' */
	a = getopt(argc, argv, "dhis:");
	if(a == -1) break;
	switch(a)
		{
        case 'd': /* debug */
			debug_flag = 1;
			break;
		case 'h':
			fprintf(stdout,\
"Usage: NewsFleX [-d] [-h] [-shared]\n\
\t-d  send debug output to console.\n\
\t-i  do not send NewsFleX ID in headers.\n\
\t-h  help (this help).\n\
\t-s  shared (for < 24 bit color map sharing).\n\
");
			exit(0);	
			break;
		case 'i':
			send_newsflex_id = 0;
			break;
		case 's':
			break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stdout, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stdout, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			exit(1);
			break;			
		default:
			break;
		}/* end switch a */
	}/* end while getopt() */

/* make the locale portable so we can do date in English in postings */
//ptr = setlocale(LC_ALL, "" );
//if(! ptr)
//	{
//	fprintf(stdout, "setlocale() failed, could not make locale portable for this program, aborting.\n");

//	exit(1);
//	}

/* get linux version */
/*
uname -a
Linux panteltje 2.4.25 #1 Fri Mar 26 18:17:35 CET 2004 i686 unknown
*/
sprintf(temp, "uname -a > %s/.NewsFleX/kernel_version", home_dir);

fptr = popen(temp, "r");
pclose(fptr);

sprintf(temp, "%s/.NewsFleX/kernel_version", home_dir); 
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr, "NewsFleX: could not open %s for read, aborting\n", temp);

	exit(1);	
	}

fscanf(fptr, "%s %s %s", dummy, dummy, temp);

fclose(fptr);

kernel_version = strsave(temp);
if(! kernel_version)
	{
	fprintf(stderr, "NewsFleX could not get kernel version, aborting\n");

	exit(1);
	}

if(debug_flag)
	{
	fprintf(stdout, "kernel version=%s\n", kernel_version);
	}

eeepc_flag = 0;
if( strstr(temp, "eeepc") != 0)
	{
	eeepc_flag = 1;
	}
	
if(debug_flag)
	{
	fprintf(stderr, "eeepc_flag=%d\n", eeepc_flag);
	}

/* no server connection yet */
socketfd = -1;

/* no command send to server yet */
command_in_progress = 0;

/*
prevent executing an empty command queue, then doing an auto disconnect
from net, if it is enabled.
That would kill other connection (such as ftp) that uses the net.
*/
process_command_queue_error_flag = 1;

/* why was this ???? */
fl_set_goodies_font(FL_BOLD_STYLE, FL_MEDIUM_SIZE);

/* 
this must be called before fl_initialize, but with home_dir set
if this fails, default colors will be used
*/
load_custom_colors();

fl_initialize(&argc, argv, 0, 0, 0);

fdui = create_form_NewsFleX();

/* set the icon */
sprintf(temp, "%s/.NewsFleX/NewsFleX.xpm", home_dir);
p = fl_read_pixmapfile(fl_root, temp, &w, &h, &mask, 0, 0, 0);
fl_set_form_icon(fdui -> NewsFleX, p, mask);

add_system_fonts();

/* show the first form */
fl_show_form(fdui -> NewsFleX, FL_PLACE_FREE, FL_FULLBORDER, PVERSION);

groups_show_selection = SHOW_ONLY_SUBSCRIBED_GROUPS;
articles_menu_selection = SUBSCRIBE;
visible_browser = GROUP_LIST_BROWSER;
news_server_status = DISCONNECTED;
news_server_mode = POSTING_ALLOWED;
mail_server_status = DISCONNECTED;
http_server_status = DISCONNECTED;
ftp_control_server_status = DISCONNECTED;
ftp_data_server_status = DISCONNECTED;

/* these values are fixed */
connect_to_http_server_timeout = 30;
connect_to_ftp_server_timeout = 30; 

groups_display_filter = strsave("");
selected_group = strsave("");
net_status = UNKNOWN;

load_newsservers();

/* news_server_name and database_name set by load_newsservers */
sprintf(temp, "%s current", news_server_name);

/* also loads general_settings and fontsizes */
select_newsserver(temp);

/*
sets fonts for buttons and articles,
this must happen after fontsizes it seems
*/
load_selected_fonts();

load_filters();

load_search_settings();

load_highlight_settings();

load_posting_periods();
select_posting_period("current");

load_subscribed_groups(USE_ALL_DATA);

select_first_group_and_article();

load_helpers();

load_urls();

refresh_screen(selected_group);

ask_question_flag = -1;

/*delete_all_message_ids()*/
load_message_ids();

/*
test if the period has changed, and ask the user if he / she wants to
move the current database for this news server to one named with the
name of the previous period.
*/
test_period_flag = 1;

//check_mail_local();

/* start external editor thread */
edit_init();

fl_do_forms();

return(0);
}/* end function main */

